#ifndef icqDialogs_H
#define icqDialogs_H


#include <qwidget.h>
#include <qtimer.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qdialog.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qlineedit.h>
#include <qtabbar.h>

#include "icqfunctions.h"
#include "icq.h"
#include "icqevent.h"



class AnimatedUserLabel : public QLabel
{
   Q_OBJECT
public:
   AnimatedUserLabel(QWidget *parent = 0);
   ~AnimatedUserLabel();
protected:
   int timerid;
   int picnum;
   virtual void timerEvent(QTimerEvent *);

public slots:
   void start();
   void stop();

};


/////////////////////////
// send message dialog box
/////////////////////////

class SendMessageDlg : public QDialog
{
   Q_OBJECT
public:
//   SendMessageDlg (ICQ *s, ICQUser *u, QWidget *parent = NULL, const char *name = NULL);
   SendMessageDlg ( ICQ *s, ICQUser *u, QWidget *parent=0, const char *name=0, bool modal=FALSE, WFlags f=0 );


   MLEditWrap *mleMessage;
   void setMoreClicked( bool );
   void setMultipleClicked( bool );

protected:

   QGroupBox *tobox, *recipientbox;
   QLabel *uinlabel, *handlelabel, *emaillabel;
   QLabel *uinline, *handleline, *emailline;
   QLabel *messagelabel;
   AnimatedUserLabel *lblAnimation;

   ICQ *server;
   QPushButton *btnSend, *btnCancel, *btnHistory, *btnMore, *btnMultiple, *btnInfo;

   QCheckBox *chkSendServer;

   QListView *recipientList;

   ICQUser *theUser;
   ICQEvent *icqEvent;

   void setupMultipleMessages( void );

protected slots:
   void clickSndServer( void );
   void doneSend( bool, ICQEvent * );
   void enableHistoryBtn(); 
   void sendMessage( void );
   void clickMore( void );
   void showHistory( void );
   void showInfo( void );
   void clickCancel( void );
   void clickMultiple( void );
public slots:
   virtual void show();
   virtual void hide();
   void killMe();

signals:
   void finished(ICQUser *, int);
   void killIt(); 
};

/////////////////////////
// history dialog box
/////////////////////////

class HistoryDlg : public QDialog
{
   Q_OBJECT
public:
   HistoryDlg (ICQUser *u, QWidget *parent = NULL, const char *name = NULL);

protected:
   void showHistory();

   QPushButton *btnCancel, *btnDeleteAll;
   MLEditWrap *mleHistory;

   ICQUser *theUser;

public slots:
   virtual void show();
   void deleteAll();
   virtual void hide();

signals:
   void finished(ICQUser *, int);
};


/////////////////////////
// read message dialog box
/////////////////////////

class ReadMsgDlg : public QDialog
{
   Q_OBJECT
public:
   ReadMsgDlg (ICQ *s, ICQUser *u, QWidget *parent = NULL, const char *name = NULL);

protected:
   HistoryDlg *dlgHistory;

   QGroupBox *frombox, *messagebox, *responsebox;
   QLabel *uinlabel, *handlelabel, *emaillabel, *datelabel, *timelabel;
   QLabel *uinline, *handleline, *emailline, *dateline, *timeline;

   ICQ *server;
   QPushButton *btnNext, *btnClose, *btnRequest, *btnHistory, *btnForward, *btnReply;
   MLEditWrap *mleRead;

   ICQUser *theUser;
   ICQEvent *icqEvent;   

public slots:
   virtual void show();

protected slots:
   void killWindow(ICQUser *, int window);
   void readMessage( void );
   void showHistory( void );
   void readNext( void );
   void reply();
   void forward();
   void requestChat();
   void enableHistoryBtn();
   void enableReplyBtn();
   void killChatRequestDlg(ICQUser *u, int window);
   void doneChatRequest(bool, ICQEvent *); 
   void incomingMessage(); 
   void clickClose();

signals:
   void finished(ICQUser *, int);
   void updatedUsers( void );
};



/////////////////////////
// send URL dialog box
/////////////////////////

class SendURLDlg : public QDialog
{
   Q_OBJECT
public:
   SendURLDlg (ICQ *s, ICQUser *u, QWidget *parent = NULL, const char *name = NULL);

   MLEditWrap *mleMessage;
protected:
   HistoryDlg *dlgHistory;

   QGroupBox *tobox;
   QLabel *uinlabel, *handlelabel, *emaillabel;
   QLabel *uinline, *handleline, *emailline;
   QLabel *messagelabel, *urllabel;

   ICQ *server;
   QPushButton *btnSend, *btnCancel, *btnMore, *btnInfo;
   QLineEdit *edtURL;

   QCheckBox *chkSendServer;

   ICQUser *theUser;
   ICQEvent *icqEvent;

   AnimatedUserLabel *lblAnimation;

protected slots:
   void clickSndServer( void );
   void doneSend(bool, ICQEvent *);
   void sendURL( void );
   void clickMore( void );

public slots:
   virtual void show();
   virtual void hide();
   void clickCancel( void );
   void showInfo( void ); 

signals:
   void finished(ICQUser *, int);
};


/////////////////////////
// chat request dialog box
/////////////////////////

class ChatRequestDlg : public QDialog
{
   Q_OBJECT
public:
   ChatRequestDlg (ICQ *s, ICQUser *u, QWidget *parent = NULL, const char *name = NULL);
private:
   QGroupBox *tobox;
   QLabel *label4;
   QLabel *label2;
   QLabel *lblNickname;
   QLabel *lblICQNumber;
   QLabel *lblEmail;
   QPushButton *btnInfo;
   QPushButton *btnJoin;
   QLabel *label3;
   QMultiLineEdit *mleSubject;
   QPushButton *btnChat;
   QPushButton *btnCancel;
   QLabel *label1;

   AnimatedUserLabel *lblAnimation;

   ICQ *server;
   ICQEvent *icqEvent;

protected:

   ICQUser *theUser;
protected slots:
   void doneChatRequest( bool, ICQEvent * );

public slots:
   virtual void show();
   virtual void hide();
   void showInfo( void );
   void requestChat();
   void clickCancel();

signals:
   void finished(ICQUser *, int);
};


/////////////////////////
// read away message dialog box
/////////////////////////

class ReadAwayMsgDlg : public QDialog
{
   Q_OBJECT
public:
   ReadAwayMsgDlg (ICQ *s, ICQUser *u, QWidget *parent = NULL, const char *name = NULL);

protected:

   ICQ *server;
   ICQUser *theUser;

   QPushButton *btnUpdate, *btnCancel;   
   MLEditWrap *mleAwayMsg;


public slots:
   virtual void show();
   virtual void hide();
   void update();

signals:
   void finished(ICQUser *, int);
};


/////////////////////////
// away message dialog box
/////////////////////////

class
AwayMessageDlg : public QDialog
{
   Q_OBJECT
public:
   AwayMessageDlg(ICQUser *u, QWidget *parent=0, char *name=0);
private:
   QMultiLineEdit *mlined1;
   QLabel *label1;
   QLabel *label3;
   QLabel *label2;
   QCheckBox *check1;
   QPushButton *btnOK, *btnMessage;
public slots:
};



/////////////////////////
//  about dialog box
/////////////////////////
 
class AboutDlg : public QDialog
{
   Q_OBJECT
public:
   AboutDlg(QWidget *parent=0, char *name=0);

private:
   QLabel *lblThanks;
   QPushButton *btnOK;
   QLabel *lblVersion;
   QLabel *lblAuthor;
   QLabel *lblTitle;
   QLabel *label1;

public slots:

signals:
 
};

/////////////////////////
// info dialog box
/////////////////////////

class UserInfoDlg : public QDialog
{
   Q_OBJECT
public:
   UserInfoDlg (ICQ *s, ICQUser *u, QWidget *parent = NULL, const char *name = NULL);

protected:
   QLineEdit *edtLastName, *edtFirstName, *edtEmail, *edtNickname, *edtUIN, *edtCurrentIP;
   QLabel *lblFirstName, *lblEmail, *lblNickname, *lblCurrentIP, *lblUIN, *lblLastName;
   AnimatedUserLabel *lblAnimation;
   QPushButton *btnUpdate, *btnDone;

   ICQUser *theUser;
   ICQ *server;

private slots:
   void updateUser();
   void updatedUserInfo(bool isOk, unsigned long uin);
public slots:
   virtual void hide();

signals:
//   void finished(ICQUser *, int);
};



/////////////////////////
// rename dialog box
/////////////////////////

class UserRenameDlg : public QDialog
{
   Q_OBJECT
public:
   UserRenameDlg (ICQUser *u, QWidget *parent = NULL, const char *name = NULL);

protected:

private:
   QPushButton *btnRename, *btnCancel;
   QLineEdit *edtNickname;
   QLabel *lblDescription;

   ICQUser *theUser;

public slots:
   void renameUser();
   virtual void hide();

signals:
   void finished(ICQUser *, int);

};



/////////////////////////
// search dialog box
/////////////////////////

class SearchUserView : public QListView
{
   Q_OBJECT
public:
   SearchUserView(QWidget *parent = NULL, char *name = NULL);
   unsigned long currentUin(void);
};


class SearchItem : public QListViewItem
{
public:
   SearchItem(struct UserStatusline &, QListView *parent);  
   unsigned long uin();
protected:
   unsigned long uinVal;
};


class ICQSearchDlg : public QDialog
{
   Q_OBJECT
public:
   ICQSearchDlg (ICQ *s, QWidget *parent = NULL, const char *name = NULL);

protected:

   AnimatedUserLabel *lblAnimation;
   QButtonGroup *grpDialog, *grpSearchButtons;
   QPushButton *btnSearch, *btnStopSearch, *btnNewSearch;

   QLineEdit *edtNick, *edtFirstName, *edtLastName, *edtEmail;
   QLabel *lblEmail, *lblNick, *lblFirstName, *lblLastName;

   SearchUserView *foundView;
   unsigned long searchSequence;

private:
   ICQ *server;
//   QTabBar *tabSearchBar;
//   QWidget *tabEmail;
   
private slots:
   void beginSearch();
   void stopSearch();
   void newSearch();
   void userFound(struct UserStatusline &, unsigned long);
   void doneSearch(unsigned short, char);   
   void addUser();

public slots:
   virtual void show();
signals:

};




#endif


