#include <qmsgbox.h> 

#include "icqgui.h"
#include "connectbutton.h"

#include <X11/Xlib.h>  // stuff for 
#include <X11/Xutil.h> // appicon generation

#ifdef USE_SCRNSAVER
#include <X11/extensions/scrnsaver.h>
#endif // USE_SCRNSAVER

#include <qpoint.h>

//pictures
#include "pics/status_online.xpm"
#include "pics/status_offline.xpm"
#include "pics/status_private.xpm"
#include "pics/status_dnd.xpm"
#include "pics/status_away.xpm"
#include "pics/status_freeforchat.xpm"
#include "pics/status_occupied.xpm"
#include "pics/status_na.xpm"    

#include "pics/null.xpm"
#include "pics/user_icon.xpm"
#include "pics/user_na.xpm"
#include "pics/user_away.xpm"
#include "pics/user_dnd.xpm"
#include "pics/user_occupied.xpm"
#include "pics/user_private.xpm"
#include "pics/user_freeforchat.xpm"

#include "pics/message.xpm"
#include "pics/file.xpm"
#include "pics/url.xpm"
#include "pics/chat.xpm"
#include "pics/history.xpm"

#include "pics/adduser.xpm"
#include "pics/renameuser.xpm"
#include "pics/removeuser.xpm"
#include "pics/addfind.xpm"
#include "pics/info.xpm"
#include "pics/shutdown.xpm"
#include "pics/icq.xpm"
#include "pics/preferences.xpm"
#include "pics/authorize.xpm"

NewLabel::NewLabel(QWidget *parent = 0) : QLabel(parent)
{

}

void NewLabel::setBold(bool isBold)
{
   QFont newFont(font());
   newFont.setBold(isBold);
   setFont(newFont);
}

void NewLabel::setItalic(bool isItalic)
{
   QFont newFont(font());
   newFont.setItalic(isItalic);
   setFont(newFont);
}

void NewLabel::setNamedColor(char *theColor)
{
   QPalette pal(palette());
   QColorGroup normal(pal.normal());
   QColorGroup newNormal(normal.foreground(), normal.background(), normal.light(), normal.dark(),
                         normal.mid(), QColor(theColor), normal.base());
   setPalette(QPalette(newNormal, pal.disabled(), newNormal));
}


void NewLabel::mouseDoubleClickEvent(QMouseEvent *)
{
   emit doubleClicked();
}


//-----ICQnixGUI::constructor------------------------------------------------------------------------
ICQnixGUI::ICQnixGUI(ICQ *theServer, QWidget *parent, const char *name) : QWidget(parent, name)
{
   // set up appicon and docking, code supplied by Mark Deneed
   WId win = winId();     // get the window
   Display *dsp = x11Display();  // get the display
   XWMHints *hints;  // hints
   XClassHint classhint;  // class hints
   classhint.res_name = "ICQnix";  // res_name
   classhint.res_class = "ICQnix";  // res_class   
   XSetClassHint(dsp, win, &classhint); // set the class hints
   hints = XGetWMHints(dsp, win);  // init hints
   hints->window_group = win;  // set set the window hint
   hints->flags = WindowGroupHint;  // set the window group hint
   XSetWMHints(dsp, win, hints);  // set the window hints for WM to use.
   XFree( hints ); 

   
   // set the icon
   setIconText("ICQNix");
   setIcon(QPixmap(online_xpm));

   // read in info from file
   char filename[1024];
   unsigned short gridLinesInt, fontCharSetInt;
   
   
   if (DEBUG_LEVEL >= 1) cout << "-> GUI configuration." << endl;
   sprintf(filename, "%s%s", BASE_DIR, "icqnix.conf");
   File icqConf(filename);

   if (icqConf.error()) { cerr << "Fatal error!  Unable to open " << filename << "." << endl; exit(1); }
   icqConf.section("appearance").key("FontSize").read(fontSize, true);
   icqConf.section("appearance").key("FontFamily").read(fontFamily, true);
   icqConf.section("appearance").key("FontCharSet").read(fontCharSetInt, true);
   switch (fontCharSetInt)
   {
   case 1: fontCharSet = QFont::Latin1; break;
   case 2: fontCharSet = QFont::Latin2; break;
   case 3: fontCharSet = QFont::Latin3; break;
   case 4: fontCharSet = QFont::Latin4; break;
   case 5: fontCharSet = QFont::Latin5; break;
   case 6: fontCharSet = QFont::Latin6; break;
   case 7: fontCharSet = QFont::Latin7; break;
   case 8: fontCharSet = QFont::Latin8; break;
   case 9: fontCharSet = QFont::Latin9; break;
   case 10: fontCharSet = QFont::KOI8R; break;
   default: fontCharSet = QFont::Latin1; break;
   }
   if (DEBUG_LEVEL >= 1) cout << "   Font: " << fontFamily << " / " << fontSize << "." << endl;
   qApp->setFont(QFont(fontFamily, fontSize, QFont::Normal, false, fontCharSet), true);

   icqConf.section("appearance").key("ColorOnline").read(colorOnline, true);
   icqConf.section("appearance").key("ColorOnlineBackground").read(colorOnlineBack, true);
   icqConf.section("appearance").key("ColorOffline").read(colorOffline, true);
   icqConf.section("appearance").key("ColorOfflineBackground").read(colorOfflineBack, true);
   icqConf.section("appearance").key("ColorAway").read(colorAway, true);
   icqConf.section("appearance").key("ColorPrivate").read(colorPrivate, true);
   icqConf.section("appearance").key("ColorPrivateBackground").read(colorPrivateBack, true); 
   icqConf.section("appearance").key("ColorAwayBackground").read(colorAwayBack, true);
   icqConf.section("appearance").key("ColorNew").read(colorNew, true);
   if (DEBUG_LEVEL >= 1) cout << "   Colors: " << colorOnline << " / " << colorOffline << " / " << colorAway << " / " << colorNew << "." << endl;

   icqConf.section("appearance").key("GridLines").read(gridLinesInt);
   gridLines = (gridLinesInt != 0);
   if (DEBUG_LEVEL >= 1) cout << "   Grid Lines " << (gridLines ? "enabled." : "disabled.") << endl;

   icqConf.section("startup").key("Logon").read(logonNow, true);
   if (DEBUG_LEVEL >= 1) cout << "   Autologon " << (logonNow == 0 ? "disabled." : "enabled.") << endl;
   icqConf.section("startup").key("AutoAway").read(autoAwayTime, true);
   if (DEBUG_LEVEL >= 1) 
      if (autoAwayTime == 0) 
         cout << "   Auto-away disabled." << endl;
      else
         cout << "   Auto-away enabled after " << autoAwayTime << " minutes." << endl;
   icqConf.section("startup").key("AutoNA").read(autoNATime, true);
   if (DEBUG_LEVEL >= 1) 
      if (autoNATime == 0) 
         cout << "   Auto-not-available disabled." << endl;
      else
         cout << "   Auto-not-available enabled after " << autoNATime << " minutes." << endl;

   
   // load up position and size from file
   if (DEBUG_LEVEL >= 1) cout << "-> Geometry configuration." << endl;
   sprintf(filename, "%s%s", BASE_DIR, "geometry.conf");
   File geoConf(filename);
   if (!geoConf.error()) 
   {
      unsigned short xPos, yPos, hVal, wVal;
      geoConf.section("geometry").key("x").read(xPos);
      geoConf.section("geometry").key("y").read(yPos);
      geoConf.section("geometry").key("h").read(hVal, true);
      geoConf.section("geometry").key("w").read(wVal, true);
      if (DEBUG_LEVEL >= 1) cout << "   Using width " << wVal << ", height " << hVal << " at (" << xPos << ", " << yPos << ")." << endl;
      setGeometry(xPos, yPos, wVal, hVal);
   }
   else 
   {
      if (DEBUG_LEVEL >= 1) cout << "   Using defaults: width 160, height 257 at (0, 0)." << endl;
      setGeometry(0, 0, 160, 257);
   }
   setMinimumSize( 100, 200 );

   // load the pixmaps

   pmOnline = new QPixmap(usericon_xpm);
   pmOffline = new QPixmap(usericon_xpm);
   pmFreeForChat = new QPixmap(userfreeforchat_xpm);
   pmAway = new QPixmap(useraway_xpm);
   pmNa = new QPixmap(userna_xpm);
   pmOccupied = new QPixmap(useroccupied_xpm);
   pmDnd = new QPixmap(userdnd_xpm);
   pmPrivate = new QPixmap(userprivate_xpm);
   pmMessage = new QPixmap(message_xpm);


// 
// this will eventually be for themes, right now it's just old code
// 
/*   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "online.ppm");
   pmOnline = new QPixmap(filename);
   if (pmOnline == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;
   
   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "freeforchat.ppm");
   pmFreeForChat = new QPixmap(filename);
   if (pmFreeForChat == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;

   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "offline.ppm");
   pmOffline = new QPixmap(filename);
   if (pmOffline == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;
   
   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "away.ppm");
   pmAway = new QPixmap(filename);
   if (pmAway == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;
   
   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "na.ppm");
   pmNa = new QPixmap(filename);
   if (pmNa == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;
   
   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "occupied.ppm");
   pmOccupied = new QPixmap(filename);
   if (pmOccupied == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;
   
   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "dnd.ppm");
   pmDnd = new QPixmap(filename);
   if (pmDnd == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;

   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "private.ppm");
   pmPrivate = new QPixmap(filename);
   if (pmPrivate == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;
   
   sprintf(filename, "%s%s%s", BASE_DIR, THEME_DIR, "message.ppm");
   pmMessage = new QPixmap(filename);
   if (pmMessage == NULL) cerr << "Warning!  Unable to open pixmap " << filename << ".  Using blank." << endl;
*/
   
   // create the child widgets
   server = theServer;
   netWin = new OutputWin;
   server->setOutputWin(netWin);
   awayMsgDlg = new AwayMsgDlg(&server->icqOwner);
   addUserDlg = new AddUserDlg(server);
   authUserDlg = new AuthUserDlg(server);
   //searchUserDlg = new SearchUserDlg(server);
   searchUserDlg = new ICQSearchDlg(server);
   aboutBox = new AboutDlg();
   
   char title[128];
   //sprintf(title, "ICQnix (%s)", server->icqOwner.alias());
   icqConf.section("owner").key("Uin").read(title, true);
   setCaption(title);

   mnuSystem = new QPopupMenu(NULL);
   mnuSystem->setFont( QFont( "", 10 ) );
   mnuSystem->insertItem("System Message History", this, SLOT(readSysMsgHistory()) );

   mnuSound = new QPopupMenu(NULL);
   mnuSound->setFont( QFont( "", 10 ) );
   mnuSound->setCheckable(true);
   mnuSound->insertItem("No Sound");
   mnuSound->insertItem("User Defined Player");
   mnuSound->insertItem("PC Speaker");
   mnuSound->setItemChecked(server->soundEnabled(), true);
   connect(mnuSound, SIGNAL(activated(int)), this, SLOT(changeSound(int)));

   mnuAdvanced = new QPopupMenu(NULL);
   mnuAdvanced->setFont( QFont( "", 10 ) );
   mnuAdvanced->insertItem("Update Contact List", server, SLOT(updateContactList()));
   mnuAdvanced->insertItem("Update All Users", server, SLOT(updateAllUsers()));
   mnuAdvanced->insertItem("Redraw User Window", this, SLOT(updateUserWin()));
      
   mnuOptions = new QPopupMenu(NULL);
   mnuOptions->setFont( QFont( "", 10 ) );
   mnuOptions->insertItem("Draw Grid Lines");
   mnuOptions->insertItem("Sound", mnuSound);
   mnuOptions->setCheckable(true);
   mnuOptions->setItemChecked(0, gridLines);
   mnuOptions->connectItem(0, this, SLOT(toggleGridLines()));
   mnuDebug = new QPopupMenu(NULL);
   mnuDebug->insertItem("No debugging information");
   mnuDebug->insertItem("Startup comments only");
   mnuDebug->insertItem("Show all packets");
   mnuDebug->setCheckable(true);
   mnuOptions->insertItem("Debug Level", mnuDebug);
   connect(mnuDebug, SIGNAL(activated(int)), this, SLOT(changeDebug(int)));
   changeDebug(DEBUG_LEVEL);

   mnuFile = new ICQPopupMenu(NULL);
   mnuFile->setFont( QFont( "", 10 ) );
   //mnuFile->insertItem(server->icqOwner.alias(), this, SLOT(callICQOwnerFunction()) );
   mnuFile->insertItem(authorize_xpm, "Authorize User", authUserDlg, SLOT(show()) );
   mnuFile->insertItem(adduser_xpm, "Add User", addUserDlg, SLOT(show()));
   mnuFile->insertSeparator();
   mnuFile->insertItem("Show Network Window", netWin, SLOT(show()));
   mnuFile->insertItem("Toggle Mini Mode", this, SLOT(miniMode()));
   mnuFile->insertItem(preferences_xpm, "Preferences", mnuOptions);
   mnuFile->insertItem("Advanced", mnuAdvanced);
   mnuFile->insertItem("About", aboutBox, SLOT(show()));
   mnuFile->insertSeparator();
   mnuFile->insertItem("Next Server", this, SLOT(nextServer()));
   mnuFile->insertSeparator();
   mnuFile->insertItem(shutdown_xpm, "Exit", qApp, SLOT(quit()));
   connect(mnuFile, SIGNAL(hideMenu()), this, SLOT(depressButton()));

   mnuStatus = new ICQPopupMenu(NULL);
   mnuStatus->setFont( QFont( "", 10 ) );
   mnuStatus->insertItem(online_xpm, "Online");
   mnuStatus->insertItem(freeforchat_xpm, "Free for Chat");
   mnuStatus->insertItem(away_xpm, "Away");
   mnuStatus->insertItem(na_xpm, "Not Available");
   mnuStatus->insertItem(occupied_xpm, "Occupied");
   mnuStatus->insertItem(dnd_xpm, "Do Not Disturb");
   mnuStatus->insertItem(private_xpm, "Private (Invisible)");
   mnuStatus->insertItem(offline_xpm, "Offline");
   connect(mnuStatus, SIGNAL(activated(int)), this, SLOT(changeStatus(int)));
   connect(mnuStatus, SIGNAL(hideMenu()), this, SLOT(depressButton()));
   
   mnuUser = new QPopupMenu(NULL);
   mnuUser->setFont( QFont( "", 10 ) );
  // connect(mnuUser, SIGNAL(aboutToShow()), this, SLOT(mnuUserClicked()) );
   //mnuUser->insertItem("Read Message");
	// SEND
   mnuUser->insertItem(message_xpm, "Message");
   mnuUser->insertItem(file_xpm, "File");
   mnuUser->insertItem(url_xpm, "Web Page Address (URL)");
   mnuUser->insertSeparator();   
   	// INVITE
   mnuUser->insertItem(chat_xpm, "ICQ Chat");
   mnuUser->insertItem(away_xpm, "Read Away Message");
   mnuUser->insertSeparator();
   mnuUser->insertItem(history_xpm, "View Message History");
   mnuUser->insertSeparator();
   mnuUser->insertItem(info_xpm, "Info"); 
   mnuUser->insertItem(renameuser_xpm, "Rename");
   mnuUser->insertItem(removeuser_xpm, "Delete", this, SLOT(removeUser()));
   mnuUser->insertSeparator();
   mnuUser->insertItem("Invisible To User");
   mnuUser->insertItem("Visible To User");
 
 
   connect (mnuUser, SIGNAL(activated(int)), this, SLOT(callICQUserFunction(int))); 
  
   char colKey[16];
   unsigned short colTitle, colWidth, colAlign, numColumns;
   vector <ColInfo> colInfo;
   icqConf.section("appearance").key("NumColumns").read(numColumns, true);
   for (unsigned short i = 1; i <= numColumns; i++) 
   {
      sprintf(colKey, "Column%dInfo", i);
      icqConf.section("appearance").key(colKey).read(colTitle, true);
      sprintf(colKey, "Column%dWidth", i);
      icqConf.section("appearance").key(colKey).read(colWidth, true);
      sprintf(colKey, "Column%dAlign", i);
      icqConf.section("appearance").key(colKey).read(colAlign, true);
      colInfo.push_back(ColInfo(colTitle, colWidth, colAlign));
   }
   
   //splUser = new QSplitter(QSplitter::Vertical, this);
   //splUser->setStyle(MotifStyle);
   //userViewOnline = new UserView(mnuUser, colInfo, true, splUser);
   //userViewOffline = new UserView(mnuUser, colInfo, false, splUser);
   userView = new UserView(mnuUser, colInfo, true, this);
   userView->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);

   statusBar = new QGroupBox(this);
   statusBar->setFrameStyle(QFrame::NoFrame ); // | QFrame::Plain );
   lblSysMsg = new NewLabel(statusBar);
   lblSysMsg->setFrameStyle(QFrame::Panel | QFrame::Sunken);
   lblSysMsg->setBold(false);
   connect( lblSysMsg, SIGNAL(doubleClicked()), this, SLOT(readSysMsg()) );
   
   cmdSearch = new QPushButton ( "Add/Find Users", statusBar );
   cmdSearch->setPixmap ( addfind_xpm );
   connect( cmdSearch, SIGNAL(clicked()), searchUserDlg, SLOT(show()) );

   cmdStatus = new PopupButton ( statusBar );
   cmdStatus->setFont( QFont( "", 10 ) );
   connect( cmdStatus, SIGNAL(clicked()), this, SLOT(statusCommand()) );
   connect( cmdStatus, SIGNAL(updateIcon(QPixmap)), this, SLOT(changeStatusIcon(QPixmap)) );

   cmdICQ = new QPushButton ( statusBar );
   cmdICQ->setToggleButton( true );
   cmdICQ->setPixmap ( icqmenu_xpm );
   cmdICQ->setFont( QFont( "Times", 18, QFont::Bold ) );  
   connect( cmdICQ, SIGNAL(clicked()), this, SLOT(icqCommand()) );

   lblMsg = new NewLabel(statusBar);
   lblMsg->setFrameStyle(QFrame::Panel | QFrame::Sunken);
   
   autoAwayTimer.start(60000);  // start the inactivity timer for auto away
   
   connect (&autoAwayTimer, SIGNAL(timeout()), this, SLOT(autoAway()));
   connect (server, SIGNAL(updatedUsers()), this, SLOT(updateUserWin()));
   connect (server, SIGNAL(updatedStatus()), this, SLOT(updateStatus()));
   connect (server, SIGNAL(doneOwnerFcn(bool, unsigned short)), this, SLOT(done(bool, unsigned short)));
   connect (server, SIGNAL(eventResult(ICQUser *, unsigned short, bool, unsigned long)), this, SLOT(eventResult(ICQUser *, unsigned short, bool, unsigned long)));
   //connect (userViewOnline, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(callDefaultICQFunction()));
   //connect (userViewOffline, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(callDefaultICQFunction()));
   connect (userView, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(callDefaultICQFunction()));
   connect (addUserDlg, SIGNAL(updatedUsers()), this, SLOT(updateUserWin()));
   
   inMiniMode = false;
   userView->showHeader(false);

   updateStatus();
   updateUserWin();
   
   resizeEvent(NULL);

   if (logonNow != 0) changeStatus(logonNow - 1);  // automatically logon if requested
}


//-----ICQnixGUI::destructor-------------------------------------------------------------------------
ICQnixGUI::~ICQnixGUI(void)
{
   if (server->icqOwner.status() != ICQ_STATUS_OFFLINE)
      server->setStatus(ICQ_STATUS_OFFLINE); //logoff first

   // save window position and size
   char buf[MAX_FILENAME_LEN];
   sprintf(buf, "%s%s", BASE_DIR, "geometry.conf");
   QFile geoConf(buf);

   geoConf.open(IO_WriteOnly);
   sprintf(buf, "[geometry]\nx = %d\ny = %d\nw = %d\nh = %d\n", x(), y(), width(), height());
   geoConf.writeBlock(buf, strlen(buf));
   
   geoConf.close();
}


//-----ICQnixGUI::resizeEvent------------------------------------------------------------------------
void ICQnixGUI::resizeEvent (QResizeEvent *)
{
   //splUser->setGeometry(1, 25, width(), height() - 81);
   userView->setGeometry(5, 2, width() - 10, height() - 82);
   userView->resizeAliasHeader();
   statusBar->setGeometry(4, height() - 75, width() - 8, 72); 

   if (width() <= 175)
   {
      lblSysMsg->setGeometry(1, 0, 60, 20);
      lblSysMsg->setText("SysMsg");
   }
   else
   {
      lblSysMsg->setGeometry(1, 0, 110, 20);
      lblSysMsg->setText("System Message");
   }    
  
   lblMsg->setGeometry((lblSysMsg->x() + lblSysMsg->width() + 4), 0, width() - (lblSysMsg->x() + lblSysMsg->width()) - 12, 20);
   cmdSearch->setGeometry(1, lblSysMsg->y() + lblSysMsg->height() + 2, width() - 9, 23);
   cmdICQ->setGeometry(1, cmdSearch->y() + cmdSearch->height() + 1, 51, 26 );
   cmdStatus->setGeometry((cmdICQ->x() + cmdICQ->width() + 3), cmdSearch->y() + cmdSearch->height() + 1, width() - (cmdICQ->x() + cmdICQ->width()) - 11, 26);



   // the main window needs to pop up whether this works or not, dammit!
}


//-----ICQnixGUI::updateUserWin----------------------------------------------------------------------
void ICQnixGUI::updateUserWin()
{
   unsigned short i;
   unsigned short msgs = 0;
   
   QPixmap *pmStatus;
   QColor userColor, backColor;
   
   //userViewOnline->clear();
   //userViewOffline->clear();
   userView->clear();

   // set the pixmap and color for each user and add them to the view
   for (i = 0; i < server->users.size(); i++)
   {
      switch (server->users[i]->status())
      {
      case ICQ_STATUS_ONLINE: 
         pmStatus = pmOnline; 
         userColor.setNamedColor(colorOnline); 
         backColor.setNamedColor(colorOnlineBack);
         break;
      case ICQ_STATUS_FREEFORCHAT:
         pmStatus = pmFreeForChat;
         userColor.setNamedColor(colorOnline);
         backColor.setNamedColor(colorOnlineBack);
         break;
      case ICQ_STATUS_AWAY: 
         pmStatus = pmAway; 
         userColor.setNamedColor(colorAway); 
         backColor.setNamedColor(colorAwayBack);
	 break;
      case ICQ_STATUS_OCCUPIED: 
         pmStatus = pmOccupied; 
         userColor.setNamedColor(colorAway); 
         backColor.setNamedColor(colorAwayBack);
         break;
      case ICQ_STATUS_DND: 
         pmStatus = pmDnd; 
         userColor.setNamedColor(colorAway); 
         backColor.setNamedColor(colorAwayBack);
         break;
      case ICQ_STATUS_PRIVATE: 
         pmStatus = pmPrivate; 
         userColor.setNamedColor(colorPrivate); 
         backColor.setNamedColor(colorPrivateBack);
         break;
      case ICQ_STATUS_NA: 
         pmStatus = pmNa; 
         userColor.setNamedColor(colorAway); 
         backColor.setNamedColor(colorAwayBack);
         break;
      case ICQ_STATUS_OFFLINE: 
         pmStatus = pmOffline; 
         userColor.setNamedColor(colorOffline); 
         backColor.setNamedColor(colorOfflineBack);
         break;
      default: 
         //pmStatus = NULL;
         pmStatus = pmOnline;
         userColor.setNamedColor(colorAway); 
         backColor.setNamedColor(colorAwayBack);
         break;
      }
      
      // special cases
      if (server->users[i]->numMessages() > 0) 
      {
         pmStatus = pmMessage;
         msgs += server->users[i]->numMessages();
      }
      if (server->users[i]->newUser()) userColor.setNamedColor(colorNew);
            
      //(void) new UserViewItem(server->users[i], i, pmStatus, userColor, backColor, gridLines, (server->users[i]->status() == ICQ_STATUS_OFFLINE ? userViewOffline : userViewOnline));
      (void) new UserViewItem(server->users[i], i, pmStatus, userColor, backColor, gridLines, userView);
   }
   
   lblSysMsg->setEnabled(server->icqOwner.numMessages() > 0);
   lblSysMsg->setBold(server->icqOwner.numMessages() > 0);

   if (msgs > 0)
   {
      if ( server->icqOwner.status() != ICQ_STATUS_OCCUPIED && 
           server->icqOwner.status() != ICQ_STATUS_DND )
      {
         switchMessagePixmap = true;
         setIcon(QPixmap(message_xpm));
         timerid = startTimer(600);      
      } else { 
         killTimer( timerid );
	 if (server->icqOwner.status() == ICQ_STATUS_OCCUPIED) setIcon(QPixmap(occupied_xpm));
	 if (server->icqOwner.status() == ICQ_STATUS_DND) setIcon(QPixmap(dnd_xpm));
      }

      char msgsText[16];
      sprintf (msgsText, "%d msg%c", msgs, (msgs == 1 ? ' ' : 's'));
      lblMsg->setText(msgsText);
   }
   else 
   {

        switch (server->icqOwner.status())
        {
        case ICQ_STATUS_ONLINE: setIcon(QPixmap(online_xpm)); break;
        case ICQ_STATUS_FREEFORCHAT: setIcon(QPixmap(freeforchat_xpm)); break;
        case ICQ_STATUS_AWAY: setIcon(QPixmap(away_xpm)); break;
        case ICQ_STATUS_NA: setIcon(QPixmap(na_xpm)); break;
        case ICQ_STATUS_OCCUPIED: setIcon(QPixmap(occupied_xpm)); break;
        case ICQ_STATUS_DND: setIcon(QPixmap(dnd_xpm)); break;
        case ICQ_STATUS_PRIVATE: setIcon(QPixmap(private_xpm)); break;
        case ICQ_STATUS_OFFLINE: setIcon(QPixmap(offline_xpm)); break;
        }


      killTimer( timerid );
      lblMsg->setText("No msgs");
   }

   //resizeEvent(NULL);
}


//-----ICQnixGUI::updateStatus-----------------------------------------------------------------------
void ICQnixGUI::updateStatus()
{  
   char *theColor = colorOffline;
   icqStatus status = stOffline;

   switch (server->icqOwner.status())
   {
   case ICQ_STATUS_ONLINE: status = stOnline; theColor = colorOnline; setIcon(QPixmap(online_xpm)); break;
   case ICQ_STATUS_FREEFORCHAT: status = stFFC; theColor = colorOnline; setIcon(QPixmap(freeforchat_xpm)); break;
   case ICQ_STATUS_AWAY: status = stAway; theColor = colorAway; setIcon(QPixmap(away_xpm)); break;
   case ICQ_STATUS_NA: status = stNA; theColor = colorAway; setIcon(QPixmap(na_xpm)); break;
   case ICQ_STATUS_OCCUPIED: status = stOccupied; theColor = colorAway; setIcon(QPixmap(occupied_xpm)); break;
   case ICQ_STATUS_DND: status = stDND; theColor = colorAway; setIcon(QPixmap(dnd_xpm)); break;
   case ICQ_STATUS_PRIVATE: status = stInvisible; theColor = colorPrivate; setIcon(QPixmap(private_xpm)); break;
   case ICQ_STATUS_OFFLINE: status = stOffline; theColor = colorOffline; setIcon(QPixmap(offline_xpm)); server->updateContactList() ; break;
   }

   updateUserWin();

   cmdStatus->setStatus(status);
}

//----ICQnixGUI::changeStatus------------------------------------------------------------------------
void ICQnixGUI::changeStatus(int index)
{
   unsigned short newStatus =  ICQ_STATUS_OFFLINE;
   
   switch(index)
   {
   case 0: newStatus = ICQ_STATUS_ONLINE; break;
   case 1: newStatus = ICQ_STATUS_FREEFORCHAT; break; 
   case 2: newStatus = ICQ_STATUS_AWAY; awayMsgDlg->show(); break;
   case 3: newStatus = ICQ_STATUS_NA; awayMsgDlg->show(); break;
   case 4: newStatus = ICQ_STATUS_OCCUPIED; awayMsgDlg->show(); break;
   case 5: newStatus = ICQ_STATUS_DND; awayMsgDlg->show(); break;
   case 6: newStatus = ICQ_STATUS_PRIVATE; break;
   case 7: newStatus = ICQ_STATUS_OFFLINE; break;
   case 8: newStatus = ICQ_STATUS_AWAY; break;  //Auto Away
   case 9: newStatus = ICQ_STATUS_NA; break; //Auto N/A
   
   default: cerr << "ICQnixGUI::changeStatus(): bad index value " << index << endl;
   }
   
   // disable combo box, flip pixmap...
   cmdStatus->setEnabled(false);
   
   // call the right function
   if (newStatus == ICQ_STATUS_OFFLINE) 
      server->logoff(false);
   else if (server->icqOwner.status() == ICQ_STATUS_OFFLINE) 
   {
      if (!server->logon(newStatus))  // unable to connect to mirabilis server
      {
         QMessageBox::critical( this, "ICQnix", "No route to host (not connected or no servers set)", QMessageBox::Ok, 0);
         cmdStatus->setEnabled(true);
      }
      cmdStatus->setStatus(stLogin);
   }
   else {
         server->setStatus(newStatus);
   }
}


//-----ICQnixGUI::callDefaultICQFunction-------------------------------------------------------------
void ICQnixGUI::callDefaultICQFunction()      { callICQFunction(-1, true);   }
void ICQnixGUI::callICQOwnerFunction()        { callICQFunction((server->icqOwner.numMessages() > 0 ? -1 : 100), false);  }
void ICQnixGUI::callICQUserFunction(int fcn)  { callICQFunction(fcn, true); }


//-----ICQnixGUI::callICQFunction--------------------------------------------------------------------
void ICQnixGUI::callICQFunction(int fcn, bool isUser)
{
   ICQUser *user = NULL;
   if (!isUser) 
      user = &server->icqOwner;
   else
      //if ((user = userViewOnline->currentUser()) == NULL)
      //   user = userViewOffline->currentUser();
      user = userView->currentUser();
   
   if (user != NULL) 
   {
      if (fcn < 0) fcn = (user->numMessages() == 0 ? 0 : 99);  // set default function to read or send depending on whether or not there are new messages

	switch (fcn)
        {
           case MENU_SEND_MESSAGE: // Send Message
   		if (user->messageDlg == NULL)
      		{      
         	   user->messageDlg = new SendMessageDlg(server, user);
         	   user->messageDlg->show();
         	   connect (user->messageDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                   //connect (user->messageDlg, SIGNAL(killIt()), this, SLOT(killICQFunction(ICQUser *, int)) ); 
            	}
            	break;
	   case MENU_SEND_FILE: // Send File 
                break;
           case MENU_SEND_URL: // Send URL
                if (user->sendURLDlg == NULL)
                {
                   user->sendURLDlg = new SendURLDlg(server, user);
                   user->sendURLDlg->show();
                   connect (user->sendURLDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                }
                break;
           case 3: // ---------
		   		break;
           case MENU_CHAT_REQUEST: // Chat Invitation
                if (user->chatReqDlg == NULL)
                {
                   user->chatReqDlg = new ChatRequestDlg(server, user);
                   user->chatReqDlg->show();
                   connect (user->chatReqDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                }
                break;
	   	  case MENU_READ_AWAY: // --------- Read Away Msg

                if (user->readAwayMsgDlg == NULL)
                {
                   user->readAwayMsgDlg = new ReadAwayMsgDlg(server, user);
                   user->readAwayMsgDlg->show();
                   connect (user->readAwayMsgDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                }
                break;
           case 6: // ---------
		   		break;
           case MENU_HISTORY: // Message History Dialog
                if (user->historyDlg == NULL)
                {
                   user->historyDlg = new HistoryDlg(user);
                   user->historyDlg->show();
                   //connect (user->historyDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                }
                break;
           case MENU_INFO: // User Info
				if (user->infoDlg == NULL)
                {
                   user->infoDlg = new UserInfoDlg(server, user);
                   user->infoDlg->show();
                   //connect (user->infoDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                }
                break;
           case MENU_RENAME_USER: // Rename User
	  	   
           		if (user->renameDlg == NULL)
                {
                   user->renameDlg = new UserRenameDlg(user);
                   user->renameDlg->show();
                   connect (user->renameDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                }
                break;
	   	   case MENU_DELETE_USER: // Delete User
           		break;
           case MENU_INVISIBLE: //Invisible to User
                user->setInvisible(!(user->invisibleList()));
                //mnuUser->setItemChecked(17, (user->invisibleList() == 0 ? false : true));
                server->sendInvisibleList();
                updateUserWin();
                break;
           case MENU_VISIBLE: //Visible to User
                user->setVisible(!(user->visibleList()));
                //mnuUser->setItemChecked(18, (user->visibleList() == 0 ? false : true));
                server->sendVisibleList();
                updateUserWin();
                break;
           case MENU_READ_MESSAGE: // Read Message  <-- temp # for now?
                if (user->readMsgDlg == NULL)
                {
                   user->readMsgDlg = new ReadMsgDlg(server, user);
                   connect (user->readMsgDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
                   connect (user->readMsgDlg, SIGNAL(updatedUsers()), this, SLOT(updateUserWin()) );
                   user->readMsgDlg->show();
                   break;
                }
           case MENU_READ_SYSTEM_MESSAGE: // Read System Messages
                {
		   break;
                }
           default: break;
        }
 
        //user->messageDlg->setup(user, fcn, mnuOptions->isItemChecked(1));
 

        //ICQUser *u;
   	//if ((u = userViewOnline->currentUser()) == NULL)
   	//   u = userViewOffline->currentUser();
   	//u = userView->currentUser();
   	//if (u == NULL) return;

      
      //else raise it to the top
   }   
}

void ICQnixGUI::killICQFunction(ICQUser *u, int window)
{
   ICQUser *user = NULL;

   user = u;
   // 0 == sendmessage, 1 == file, 2 == url, 4 == chat, 6 == history, 8 == info, 9 == rename, 99 == readmsg
   switch( window )
   {
      case 0: if (user->messageDlg != NULL)
   	          {
                 delete user->messageDlg;
      	         user->messageDlg = NULL;
   	          }  break;
      case 1: break;
      case 2: if (user->sendURLDlg != NULL)
              {
                 delete user->sendURLDlg;
                 user->sendURLDlg = NULL;
              }  break;
      case 4: if (u->chatReqDlg != NULL)
              {
                 delete user->chatReqDlg;
                 user->chatReqDlg = NULL;
              } break;
      case 6: /*if (user->historyDlg != NULL)
   	          {
      	         delete user->historyDlg;
      	         user->historyDlg = NULL;
   	          }  */break;
     case 7: if (user->readAwayMsgDlg != NULL)
              {
                 delete user->readAwayMsgDlg;
                 user->readAwayMsgDlg = NULL;
              }  break;
	 case 8: /*if (user->infoDlg != NULL)
              {
                 delete user->infoDlg;
                 user->infoDlg = NULL;
              }*/  break;
	 case 9: if (user->renameDlg != NULL)
              {
                 delete user->renameDlg;
                 user->renameDlg = NULL;
                 updateUserWin();
              }  break;
     case 99: if (user->readMsgDlg != NULL)
	      	  {
	             delete user->readMsgDlg;
	             user->readMsgDlg = NULL;
              }  break;
     default: break;
   }
}


//-----ICQnixGUI::done-------------------------------------------------------------------------------
void ICQnixGUI::done(bool isOk, unsigned short cmd)
{
   updateStatus();
   switch (cmd) 
   {
   case ICQ_CMDxSND_LOGON: 
      cmdStatus->setEnabled(true);
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error logging on.", QMessageBox::Ok, 0);
      break;
   case ICQ_CMDxSND_LOGOFF: 
      cmdStatus->setEnabled(true);
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error logging off.", QMessageBox::Ok, 0);
      break;
   case ICQ_CMDxSND_SETxSTATUS: 
      cmdStatus->setEnabled(true);
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error changing status.", QMessageBox::Ok, 0);
      break;
   case ICQ_CMDxSND_USERxLIST:
      updateUserWin();
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error retrieving contact list.", QMessageBox::Ok, 0);
      break;
   case ICQ_CMDxSND_SYSxMSGxREQ:
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error retrieving system messages.", QMessageBox::Ok, 0);
      break;
   case ICQ_CMDxSND_PING:
      //if (!isOk) QMessageBox::critical( this, "ICQnix", "Error pinging Mirabilis, see Network Window for details.", QMessageBox::Ok, 0);
      break;
   case ICQ_CMDxSND_SYSxMSGxDONExACK:
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error acknowledging system messages.", QMessageBox::Ok, 0);
      break;
   case ICQ_CMDxSND_USERxADD:
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error informing Mirabilis of new user.", QMessageBox::Ok, 0);
      break;   
   case ICQ_CMDxSND_AUTHORIZE:
      if (!isOk) QMessageBox::critical( this, "ICQnix", "Error sending authorization.", QMessageBox::Ok, 0);
      else QMessageBox::information( this, "ICQnix", "Authorization granted.", QMessageBox::Ok, 0);
      break;   
   default:
      break;
   }
}



//-----ICQnixGUI::removeUser-------------------------------------------------------------------------
void ICQnixGUI::removeUser()
{
   ICQUser *u;
   //if ((u = userViewOnline->currentUser()) == NULL)
   //   u = userViewOffline->currentUser();
   u = userView->currentUser();
   if (u == NULL) return;
   char warning[256];
   sprintf(warning, "Are you sure you want to remove\n%s (%ld)\nfrom your contact list?", u->alias(), u->uin());
   if( QMessageBox::warning( this, "ICQnix", warning, QMessageBox::Ok, QMessageBox::Cancel) == 1)
      server->removeUser(u);
}


//-----ICQnixGUI::toggleGridLines--------------------------------------------------------------------
void ICQnixGUI::toggleGridLines()
{
   mnuOptions->setItemChecked(0, !mnuOptions->isItemChecked(0));
   gridLines = mnuOptions->isItemChecked(0);
   updateUserWin();
}



//-----ICQnixGUI::changeSound------------------------------------------------------------------------
void ICQnixGUI::changeSound(int newSound)
{
   mnuSound->setItemChecked(0, false);
   mnuSound->setItemChecked(1, false);
   mnuSound->setItemChecked(2, false);
   mnuSound->setItemChecked(newSound, true);
   
   server->setSoundEnabled(newSound);
}


//-----ICQnixGUI::changeDebug------------------------------------------------------------------------
void ICQnixGUI::changeDebug(int newLevel)
{
   DEBUG_LEVEL = newLevel;  // set the global debug level
   mnuDebug->setItemChecked(0, false);
   mnuDebug->setItemChecked(1, false);
   mnuDebug->setItemChecked(2, false);
   mnuDebug->setItemChecked(newLevel, true);
}


//-----ICQnixGUI::nextServer-------------------------------------------------------------------------
void ICQnixGUI::nextServer()
{
   if (!server->nextServer())
      QMessageBox::critical( this, "ICQnix", "Error setting destination to next server.\nTry another.", QMessageBox::Ok, 0);
}


//-----ICQnixGUI::miniMode---------------------------------------------------------------------------
void ICQnixGUI::miniMode()
{
   static unsigned short oldH;
   
   if (inMiniMode) 
   {
      userView->show();
      //splUser->show();
      setMaximumHeight(2048);
      resize(width(), oldH);
      setMinimumHeight(100);
   }
   else
   {
      userView->hide();
      //splUser->hide();
      oldH = height();
      setMinimumHeight(statusBar->height() + 5);
      resize(width(), statusBar->height() + 5);
      setMaximumHeight(statusBar->height() + 5);
   }
   inMiniMode = !inMiniMode;
}


//-----ICQnixGUI::autoAway-----------------------------------------------------------
void ICQnixGUI::autoAway()
{
   Time idleTime = 0;  /* millisecs since last input event */
   //static bool goneAutoAway = false;

   unsigned short status = server->icqOwner.status();
   if (status != ICQ_STATUS_ONLINE && status != ICQ_STATUS_AWAY && status != ICQ_STATUS_FREEFORCHAT)
      return;

#ifdef USE_SCRNSAVER
   static XScreenSaverInfo *mit_info = 0;
   if (!mit_info) mit_info = XScreenSaverAllocInfo ();
   XScreenSaverQueryInfo (x11Display(), DefaultRootWindow (x11Display()), mit_info);
   idleTime = mit_info->idle;
#endif // USE_SCRNSAVER

   if ((unsigned long)idleTime > (unsigned long)autoNATime * 60000 && autoNATime != 0)
      changeStatus(9);   // set na mode
   else if ((unsigned long)idleTime > (unsigned long)autoAwayTime * 60000 && autoAwayTime != 0)
      changeStatus(8);  // go into away mode
}


//-----ICQnixGUI::statusCommand-----------------------------------------------------------
void ICQnixGUI::statusCommand()
{
  mnuStatus->move(-1000,-1000);
  mnuStatus->show();
  QPoint pos = cmdStatus->mapToGlobal(QPoint(0,0));
  pos.setY(pos.y()-mnuStatus->height());
  mnuStatus->popup(pos);
}

//-----ICQnixGUI::icqCommand----------------------------------------------------------- 
void ICQnixGUI::icqCommand()
{
  mnuFile->move(-1000,-1000);
  mnuFile->show();
  QPoint pos = cmdICQ->mapToGlobal(QPoint(0,0));
  pos.setY(pos.y()-mnuFile->height());
  mnuFile->popup(pos);
  cmdICQ->setOn(true);
}

void ICQnixGUI::depressButton()
{
  cmdICQ->setOn(false);
  cmdStatus->setOn(false);
}


//-----ICQnixGUI::eventResult------------------------------------------------------------------------
void ICQnixGUI::eventResult(ICQUser *u, unsigned short cmd, bool accepted, unsigned long port)
{
   char result[128];
   switch (cmd) 
   {
   case ICQ_CMDxTCP_CHAT:
      if (accepted) 
      {
         ChatDlg *chatDlg = new ChatDlg(u, &(server->icqOwner), false, port);
         chatDlg->show();
      }
      else
      {
         sprintf(result, "Chat request to %s refused.", u->alias());
         QMessageBox::information(this, "ICQnix", result, QMessageBox::Ok, 0);
      }
      break;
   case ICQ_CMDxTCP_FILE:
      sprintf(result, "File transfer to %s %s.", u->alias(), accepted ? "accepted" : "refused");
      QMessageBox::information(this, "ICQnix", result, QMessageBox::Ok, 0);
      break;
   default:
      break;
   }
}


void ICQnixGUI::timerEvent(QTimerEvent *te)
{
   if(te->timerId()==timerid)
   {
      if (switchMessagePixmap == true)
      {
        switchMessagePixmap = false;
        setIcon(QPixmap(null_xpm)); 
        
      }
      else
      {     
 		switchMessagePixmap = true; 
    	setIcon(QPixmap(message_xpm));
        
//        for (i = 0; i < server->users.size() ; i++)
//        {
//            UserView->setItemPixmap(i, message_xpm)

//      pmStatus = pmMessage;
//      if (server->users[i]->newUser()) userColor.setNamedColor(colorNew);

//      (void) new UserViewItem(server->users[i], i, pmStatus, userColor, backColor, gridLines, userView);
//        }
      }
   }

}

void ICQnixGUI::readSysMsg( void )
{
   ICQUser *user = NULL;
   user = &server->icqOwner;

   if (user->numMessages() > 0)
   {
      if (user->readMsgDlg == NULL)
      {
         user->readMsgDlg = new ReadMsgDlg(server, user);
         connect (user->readMsgDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
         connect (user->readMsgDlg, SIGNAL(updatedUsers()), this, SLOT(updateUserWin()) );
         user->readMsgDlg->show();
      }
   }
}

void ICQnixGUI::readSysMsgHistory( void )
{
   ICQUser *user = NULL;
   user = &server->icqOwner;

   if (user->historyDlg == NULL)
   {
      user->historyDlg = new HistoryDlg(user);
      user->historyDlg->show();
      connect (user->historyDlg, SIGNAL(finished(ICQUser *, int)), this, SLOT(killICQFunction(ICQUser *, int)) );
      // user->historyDlg->loadSystemMessageHistory();
   }   
}

void ICQnixGUI::changeStatusIcon( QPixmap newIcon )
{
	if ( server->icqOwner.numMessages() < 1 ) // hmm...doesn't work.  also have to fix it at another place in this file
		setIcon( QPixmap(newIcon) );
}
