#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>

#include "icq-defines.h"
#include "icqnix.h"

#ifdef USE_KDE
ICQnix::ICQnix(int argc, char **argv) : KApplication(argc, argv)
#else
ICQnix::ICQnix(int argc, char **argv) : QApplication(argc, argv)
#endif
{
   DEBUG_LEVEL = 0;
   quitNowVal = false;
   icq = NULL;
   icqgui = NULL;

   // parse command line for arguments
   bool baseDirSet = false;   
   while(true)
   {
      int i = getopt(argc, argv, "hd:b:");
      if (i < 0) break;
      switch (i)
      {
      case 'h':  // help
	      usage();
	      quitNowVal = true; 
         return;
	   case 'b':  // base directory
	      sprintf(BASE_DIR, "%s/", optarg);
	      baseDirSet = true;
         break;
      case 'd':  // DEBUG_LEVEL
         DEBUG_LEVEL = atol(optarg);
         break;
      }
   }
   
   // if no base directory set on the command line then get it from HOME
   if (!baseDirSet)
   {
      char *home;
      if ((home = getenv("HOME")) == NULL)
      {
         cerr << "ICQnix: $HOME not set.\n";
	      quitNowVal = true; 
         return;
      }
      sprintf(BASE_DIR, "%s/%s/", home, DEFAULT_HOME_DIR);
   }
   
   // check if user has conf files installed
   if (access(BASE_DIR, F_OK) < 0)  // BASE_DIR does not exist
   {
      if (!installBaseDir())
      {
      //      cerr << "You must run icqnix.inst to install the data and config files to " << BASE_DIR << "." << endl;
         quitNowVal = true; 
         return;
      }
      cout << "Done installation, continuing load." << endl << endl;
   }
   
   setStyle(WindowsStyle);
   // create the main widgets
   if (DEBUG_LEVEL >= 1) cout << "\n=> Setting up ICQnix daemon." << endl;
   icq = new ICQ;
   if (DEBUG_LEVEL >= 1) cout << "\n=> Setting up ICQnix GUI." << endl;
   icqgui = new ICQnixGUI(icq);
   if (DEBUG_LEVEL >= 1) cout << "\n=> Done." << endl << endl;
   
   setMainWidget(icqgui);
   icqgui->show();
   
#ifdef USE_KDE
   kicq = new Kicq;   
   KWM::setDockWindow(kicq->winId());
   kicq->show();
   kicq->reset();
   connect(icqgui, SIGNAL(changeDockStatus(unsigned short int)), kicq,SLOT(change_dock_status(unsigned short int)));
   connect(kicq, SIGNAL(change_status_from_dock(int)), icqgui, SLOT(changeStatus(int)));
   connect(kicq, SIGNAL(lowericq()), icqgui, SLOT(lower()));
   connect(kicq, SIGNAL(raiseicq()), icqgui, SLOT(raise()));
#endif 
}


ICQnix::~ICQnix(void)
{
   if (icqgui != NULL) delete icqgui;
   if (icq != NULL) delete icq;
}


void ICQnix::usage(void)
{
   cerr << PACKAGE << " version " << VERSION << "." << endl
        << "Usage:  ICQnix [-h] [-d #] [-b configdir]" << endl << endl
        << " -h            this help screen" << endl
        << " -d #          extra debugging info, # is level of information:" << endl
        << "                  1  show starting up information" << endl
        << "                  2  dump every sent packet" << endl
        << " -b configdir  set the base directory for the config and data files" << endl;
}


bool ICQnix::installBaseDir(void)
{   

/*   // save them in icqnix.conf
   cout << "Saving info...";
   sprintf (cmd, "%sconf/icqnix.conf", BASE_DIR);
   File icqConf(cmd);
   icqConf.section("owner").key("Alias").write(alias);
   icqConf.section("owner").key("Password").write(password);
   icqConf.section("owner").key("Uin").write(uin);
   cout << "done." << endl;

*/
   if (system ("icqnix.install") )
      return(true);
   else
       return(false);

}



bool ICQnix::quitNow(void)
{
   return (quitNowVal);
}
